package com.upscapesoft.instasaverapp.activities

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.app.ActivityManager
import android.app.ProgressDialog
import android.content.ActivityNotFoundException
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.provider.Settings
import android.util.Log
import android.view.View
import android.view.ViewGroup.LayoutParams
import android.view.WindowManager
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.PermissionChecker
import androidx.core.view.GravityCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.viewpager.widget.ViewPager
import com.example.awesomedialog.*
import com.google.android.material.snackbar.Snackbar
import com.hbisoft.pickit.PickiT
import com.hbisoft.pickit.PickiTCallbacks
import com.upscapesoft.instasaverapp.R
import com.upscapesoft.instasaverapp.adapters.FragmentsPagerAdapter
import com.upscapesoft.instasaverapp.data.Constants
import com.upscapesoft.instasaverapp.databinding.ActivityMainBinding
import com.upscapesoft.instasaverapp.helper.*
import com.upscapesoft.instasaverapp.interfaces.GetData
import com.upscapesoft.instasaverapp.interfaces.OnClick
import com.upscapesoft.instasaverapp.services.ActiveService
import com.upscapesoft.instasaverapp.services.DownloadService
import com.upscapesoft.instasaverapp.utils.ThemeSettings
import com.upscapesoft.instasaverapp.utils.Utils
import com.upscapesoft.instasaverapp.utils.Utils.Companion.getColorFromAttr
import com.vimalcvs.switchdn.DayNightSwitchAnimListener
import org.greenrobot.eventbus.Subscribe
import java.io.File

class MainActivity : AppCompatActivity(), PickiTCallbacks {

    private var activity: Activity? = null
    private lateinit var binding: ActivityMainBinding

    private var fragmentsPagerAdapter: FragmentsPagerAdapter? = null
    private var method: Method? = null
    private var inputMethodManager: InputMethodManager? = null
    private var pickiT: PickiT? = null
    private var uri: Uri? = null
    private var backPressedTime: Long = 0

    private var permissionsList = arrayOf(
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE
    )

    @RequiresApi(Build.VERSION_CODES.M)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)
        window.statusBarColor = resources.getColor(R.color.background_color)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.appDrawerLayout)
        activity = this

        contentViews()
        appDrawer()
        lightDarkMode()

    }

    @RequiresApi(Build.VERSION_CODES.M)
    private fun contentViews() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            when {
                ContextCompat.checkSelfPermission(
                    activity!!, Manifest.permission.POST_NOTIFICATIONS
                ) == PackageManager.PERMISSION_GRANTED -> {
                    Log.e("TAG", "onCreate: PERMISSION GRANTED")
                }
                shouldShowRequestPermissionRationale(Manifest.permission.POST_NOTIFICATIONS) -> {
                    Snackbar.make(
                        activity!!.findViewById(R.id.root),
                        "Notification permission access denied. Please enable them in apps settings!",
                        Snackbar.LENGTH_LONG
                    ).setAction("Settings") {
                        val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        val uri: Uri = Uri.fromParts("package", packageName, null)
                        intent.data = uri
                        startActivity(intent)
                    }.show()
                }
                else -> {
                    requestPermissionLauncher.launch(
                        Manifest.permission.POST_NOTIFICATIONS
                    )
                }
            }

        }

        if (Utils.hasPermissions(this, *permissionsList)) {
            ActivityCompat.requestPermissions(this, Utils.permissions, Utils.permissionRequest)
        }

        PremiumActivity.purchaseValueFromPref = preferenceObject.getBoolean(PremiumActivity.PURCHASE_KEY, false)
        if (PremiumActivity.purchaseValueFromPref as Boolean) {
            binding.container.bannerContainer.visibility = View.GONE
        } else {
            /*Admob*/
            AdController.loadBannerAd(activity, binding.container.bannerContainer)
            binding.container.bannerContainer.visibility = View.VISIBLE
        }

        GlobalBus.getBus().register(this)
        pickiT = PickiT(this, this)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            val sharedPreferences = getSharedPreferences(packageName, 0)
            if (sharedPreferences.getString("pathSaved", "")!!.length != 0) {
                uri = Uri.parse(sharedPreferences.getString("pathSaved", ""))
            } else {
                startActivity(Intent(this@MainActivity, PermissionActivity::class.java))
            }
        } else {
            if (ActivityCompat.checkSelfPermission(
                    this@MainActivity,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE
                ) == PackageManager.PERMISSION_GRANTED
            ) {
                val directory = File(
                    Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)
                        .toString() + File.separator + resources.getString(R.string.app_name) + File.separator
                )
                if (!directory.exists()) {
                    directory.mkdir()
                }
            } else {
                startActivity(Intent(this@MainActivity, PermissionActivity::class.java))
            }
        }

        inputMethodManager = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN)

        fragmentsPagerAdapter =
            FragmentsPagerAdapter(
                this,
                supportFragmentManager
            )

        binding.viewPager.adapter = fragmentsPagerAdapter

        binding.viewPager.currentItem = 0
        binding.videosTabImg.setColorFilter(getColor(R.color.tint_color))
        binding.videosTabTxt.setTextColor(getColor(R.color.tint_color))

        binding.imagesTabImg.setColorFilter(getColorFromAttr(R.attr.text_color))
        binding.imagesTabTxt.setTextColor(getColorFromAttr(R.attr.text_color))

        binding.videosTab.setOnClickListener {
            binding.viewPager.currentItem = 0
            binding.videosTabImg.setColorFilter(getColor(R.color.tint_color))
            binding.videosTabTxt.setTextColor(getColor(R.color.tint_color))

            binding.imagesTabImg.setColorFilter(getColorFromAttr(R.attr.text_color))
            binding.imagesTabTxt.setTextColor(getColorFromAttr(R.attr.text_color))
        }

        binding.imagesTab.setOnClickListener {
            binding.viewPager.currentItem = 1
            binding.videosTabImg.setColorFilter(getColorFromAttr(R.attr.text_color))
            binding.videosTabTxt.setTextColor(getColorFromAttr(R.attr.text_color))

            binding.imagesTabImg.setColorFilter(getColor(R.color.tint_color))
            binding.imagesTabTxt.setTextColor(getColor(R.color.tint_color))
        }

        binding.viewPager.addOnPageChangeListener(object: ViewPager.OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
                Log.d("TAG", "PageScrolled")
            }

            @SuppressLint("SetTextI18n")
            override fun onPageSelected(position: Int) {
                if (position == 0) {
                    binding.videosTabImg.setColorFilter(getColor(R.color.tint_color))
                    binding.videosTabTxt.setTextColor(getColor(R.color.tint_color))

                    binding.imagesTabImg.setColorFilter(getColorFromAttr(R.attr.text_color))
                    binding.imagesTabTxt.setTextColor(getColorFromAttr(R.attr.text_color))
                } else if (position == 1) {
                    binding.videosTabImg.setColorFilter(getColorFromAttr(R.attr.text_color))
                    binding.videosTabTxt.setTextColor(getColorFromAttr(R.attr.text_color))

                    binding.imagesTabImg.setColorFilter(getColor(R.color.tint_color))
                    binding.imagesTabTxt.setTextColor(getColor(R.color.tint_color))
                }

            }

            override fun onPageScrollStateChanged(state: Int) {
                Log.d("TAG", "OnPageScrollStateChanged")
            }

        })

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val params: LayoutParams = binding.header.layoutParams
            params.height = 610
            binding.header.layoutParams = params
            binding.autoDownloadLay.visibility = View.GONE
        }

        method = Method(this@MainActivity, OnClick { position, type, data ->
            if (type.equals("getData")) {
                val progressDialog = ProgressDialog(this@MainActivity)
                progressDialog.setMessage("Loading...")
                progressDialog.setCancelable(false)
                progressDialog.show()
                val findData =
                    FindData(applicationContext, GetData { linkList, message, isData ->
                        if (isData) {
                            if (linkList.size != 0) {
                                Constants.downloadArray.clear()
                                Constants.downloadArray.addAll(linkList)
                                val intent = Intent(this@MainActivity, DownloadService::class.java)
                                intent.action = "com.download.action.START"
                                startService(intent)
                            } else {
                                Log.d("TAG", "No data found")
                            }
                        } else {
                            Log.d("TAG", message)
                        }
                        binding.inputURl.setText("")
                        progressDialog.dismiss()
                    } as GetData?)
                findData.data(data)
            }
        } as OnClick?)

        val mSharedPref: SharedPreferences = getSharedPreferences("save", MODE_PRIVATE)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            binding.autoDownloadSwitch.isChecked = mSharedPref.getBoolean("value", false)
        } else {
            binding.autoDownloadSwitch.isChecked = mSharedPref.getBoolean("value", true)
        }
        if (binding.autoDownloadSwitch.isChecked) {
            appServiceStart()
        } else if (!binding.autoDownloadSwitch.isChecked) {
            appServiceStop()
        }
        binding.autoDownloadSwitch.setOnClickListener {
            if (binding.autoDownloadSwitch.isChecked) {
                val editor: SharedPreferences.Editor =
                    getSharedPreferences("save", MODE_PRIVATE).edit()
                editor.putBoolean("value", true)
                editor.apply()
                binding.autoDownloadSwitch.isChecked = true
                appServiceStart()
            } else {
                val editor: SharedPreferences.Editor =
                    getSharedPreferences("save", MODE_PRIVATE).edit()
                editor.putBoolean("value", false)
                editor.apply()
                binding.autoDownloadSwitch.isChecked = false
                appServiceStop()
            }
        }

        binding.downloadBtn.setOnClickListener(View.OnClickListener { v: View? ->
            val url: String = binding.inputURl.text.toString()
            binding.inputURl.clearFocus()
            inputMethodManager!!.hideSoftInputFromWindow(binding.inputURl.windowToken, 0)
            if (url != "") {
                method!!.onClick(0, "getData", url)
            } else if (binding.inputURl.text.toString().contains("instagram") || binding.inputURl.text.toString().contains("")) {
                Toast.makeText(applicationContext, "Please enter a valid URL", Toast.LENGTH_LONG).show()
            }
        })

        binding.appMenuBtn.setOnClickListener {
            openAppDrawer()
        }

        binding.instagramBtn.setOnClickListener {
            val pm = activity?.packageManager
            AwesomeDialog.build(activity!!)
                .title(
                    "Instagram App?",
                    titleColor = ContextCompat.getColor(activity!!, android.R.color.white)
                )
                .body(
                    "Open the Instagram app",
                    color = ContextCompat.getColor(activity!!, android.R.color.white)
                )
                .icon(R.drawable.ic_instagram)
                .background(R.drawable.rounded_bg)
                .onPositive(
                    "Instagram",
                    buttonBackgroundColor = R.drawable.rounded_bg_white,
                    textColor = ContextCompat.getColor(activity!!, R.color.text_color)
                ) {
                    try {
                        if (Utils.isPackageInstalled("com.instagram.android", pm!!)) {
                            startActivity(packageManager.getLaunchIntentForPackage("com.instagram.android"))
                        } else {
                           Toast.makeText(applicationContext, "Instagram app is not installed. Please install!", Toast.LENGTH_LONG).show()
                        }

                    } catch (e: ActivityNotFoundException) {
                        Toast.makeText(applicationContext, "Instagram app is not installed. Please install!", Toast.LENGTH_LONG).show()
                    }

                }
                .onNegative(
                    "Dismiss",
                    buttonBackgroundColor = R.drawable.rounded_bg_white,
                    textColor = ContextCompat.getColor(activity!!, R.color.text_color)
                ) {
                    AwesomeDialog.build(activity!!).dismiss()

                }

        }

    }

    private fun appDrawer() {
        binding.appDrawerLayout.setViewScale(
            GravityCompat.START,
            0.9f
        ) //set height scale for main view (0f to 1f)
        binding.appDrawerLayout.setViewElevation(
            GravityCompat.START,
            30f
        ) //set main view elevation when drawer open (dimension)
        binding.appDrawerLayout.setViewScrimColor(
            GravityCompat.START,
            ContextCompat.getColor(this, R.color.primary)
        ) //set drawer overlay color (color)
        binding.appDrawerLayout.drawerElevation = 30f
        binding.appDrawerLayout.setRadius(GravityCompat.START, 25f)
        binding.appDrawerLayout.setViewRotation(GravityCompat.START, 0f)
        binding.appDrawerLayout.closeDrawer(GravityCompat.START)
        binding.appDrawerLayout.addDrawerListener(object : DrawerLayout.DrawerListener {
            override fun onDrawerSlide(drawerView: View, slideOffset: Float) {}
            override fun onDrawerOpened(drawerView: View) {}
            override fun onDrawerClosed(drawerView: View) {}
            override fun onDrawerStateChanged(newState: Int) {}
        })
        initAppDrawer()
    }

    private fun closeAppDrawer() {
        binding.appDrawerLayout.closeDrawer(
            GravityCompat.START
        )
    }

    private fun openAppDrawer() {
        binding.appDrawerLayout.openDrawer(
            GravityCompat.START
        )
    }

    private fun initAppDrawer() {
        binding.navMenuDrawer.navHomeBtn.setOnClickListener {
            closeAppDrawer()
        }
        binding.navMenuDrawer.navInAppPurchasesBtn.setOnClickListener {
            closeAppDrawer()
            startActivity(Intent(this@MainActivity, PremiumActivity::class.java))
        }
        binding.navMenuDrawer.navShareBtn.setOnClickListener {
            closeAppDrawer()
            Utils.shareApp(activity!!, resources.getString(R.string.app_name))
        }
        binding.navMenuDrawer.navRateBtn.setOnClickListener {
            closeAppDrawer()
            Utils.rateApp(activity!!)
        }
        binding.navMenuDrawer.navPrivacyPolicyBtn.setOnClickListener {
            closeAppDrawer()
            Utils.openUrl(activity!!, Constants.PRIVACY_POLICY_URL)
        }
        binding.navMenuDrawer.navAboutBtn.setOnClickListener {
            closeAppDrawer()
            startActivity(Intent(this@MainActivity, AboutActivity::class.java))
        }
        binding.navMenuDrawer.navMoreBtn.setOnClickListener {
            closeAppDrawer()
            Utils.moreApps(activity!!)
        }

    }

    @Subscribe
    fun getService(serviceNotify: Events.ServiceNotify?) {
        if (binding.autoDownloadSwitch != null) {
            binding.autoDownloadSwitch.isChecked = false
        }
    }

    override fun onDestroy() {
        binding.autoDownloadSwitch.isChecked = false
        GlobalBus.getBus().unregister(this)
        super.onDestroy()
    }

    private fun appServiceStart() {
        val intent = Intent(applicationContext, ActiveService::class.java)
        intent.action = "com.action.serviceStart"
        startService(intent)
    }

    private fun appServiceStop() {
        val intent = Intent(applicationContext, ActiveService::class.java)
        intent.action = "com.action.serviceStop"
        startService(intent)
    }

    private fun isMyServiceRunning(): Boolean {
        val manager = (getSystemService(ACTIVITY_SERVICE) as ActivityManager)
        for (service in manager.getRunningServices(Int.MAX_VALUE)) {
            if (ActiveService::class.java.name == service.service.className) {
                return true
            }
        }
        return false
    }

    override fun PickiTonStartListener() {}
    override fun PickiTonProgressUpdate(progress: Int) {}
    override fun PickiTonCompleteListener(
        path: String,
        wasDriveFile: Boolean,
        wasUnknownProvider: Boolean,
        wasSuccessful: Boolean,
        Reason: String
    ) {
    }

    private fun lightDarkMode() {
        binding.navMenuDrawer.dayNightSwitch.setDuration(450)
        binding.navMenuDrawer.dayNightSwitch.setIsNight(ThemeSettings.getInstance(this)!!.nightMode)
        binding.navMenuDrawer.dayNightSwitch.setListener { isNight ->
            closeAppDrawer()
            Handler().postDelayed({
                ThemeSettings.getInstance(this@MainActivity)!!.nightMode = isNight
                ThemeSettings.getInstance(this@MainActivity)!!.refreshTheme()
            }, 500)

        }
        binding.navMenuDrawer.dayNightSwitch.setAnimListener(object :
            DayNightSwitchAnimListener {
            override fun onAnimEnd() {
                finish()
                val intent = Intent(this@MainActivity, MainActivity::class.java)

                startActivity(intent)
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out)
            }

            override fun onAnimValueChanged(v: Float) {}
            override fun onAnimStart() {}
        })

    }

    private val requestPermissionLauncher = registerForActivityResult(
        ActivityResultContracts.RequestPermission()
    ) { isGranted: Boolean ->
        if (isGranted) {
            Log.d("TAG", isGranted.toString())
        } else {
            //
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == Utils.permissionRequest) {
            if (Utils.hasPermissions(this, *permissionsList)) {
                ActivityCompat.requestPermissions(this, Utils.permissions, Utils.permissionRequest)
            }

        }

        if (grantResults[0] == PermissionChecker.PERMISSION_DENIED && grantResults[1] == PermissionChecker.PERMISSION_DENIED) {
            Toast.makeText(applicationContext, "Permissions denied! Please enable them in the App Permissions page", Toast.LENGTH_LONG).show()
            val mIntent = Intent()
            val mUri = Uri.fromParts("package", packageName, null)
            mIntent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
            mIntent.data = mUri
            startActivity(mIntent)
        }

    }

    private val preferenceObject: SharedPreferences
        get() = applicationContext.getSharedPreferences(PremiumActivity.PREF_FILE, 0)

    override fun onBackPressed() {
        when {
            binding.appDrawerLayout.isDrawerOpen(GravityCompat.START) -> {
                closeAppDrawer()
            }
            backPressedTime + 2000 > System.currentTimeMillis() -> {
                super.onBackPressed()
                finish()
            }
            else -> {
                Toast.makeText(applicationContext, "Please click BACK again to exit", Toast.LENGTH_LONG).show()
            }
        }
        backPressedTime = System.currentTimeMillis()

    }

}