package com.upscapesoft.instasaverapp.activities

import android.app.Activity
import android.app.WallpaperManager
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import androidx.viewpager.widget.ViewPager
import com.theartofdev.edmodo.cropper.CropImage
import com.upscapesoft.instasaverapp.adapters.PreviewAdapter
import com.upscapesoft.instasaverapp.data.Constants
import com.upscapesoft.instasaverapp.databinding.ActivityPreviewBinding
import com.upscapesoft.instasaverapp.helper.AdController
import com.upscapesoft.instasaverapp.helper.Events
import com.upscapesoft.instasaverapp.helper.GlobalBus
import com.upscapesoft.instasaverapp.helper.Method
import com.upscapesoft.instasaverapp.utils.Utils
import org.greenrobot.eventbus.Subscribe
import java.io.File
import java.io.IOException

class PreviewActivity : AppCompatActivity() {

    private var activity: Activity? = null
    private lateinit var binding: ActivityPreviewBinding

    private var position = 0
    private var previewAdapter: PreviewAdapter? = null
    private var type: String? = null
    private var method: Method? = null
    private var showArray: MutableList<File>? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityPreviewBinding.inflate(layoutInflater)
        setContentView(binding.root)
        activity = this

        contentViews()

    }

    private fun contentViews() {
        PremiumActivity.purchaseValueFromPref = preferenceObject.getBoolean(PremiumActivity.PURCHASE_KEY, false)
        if (PremiumActivity.purchaseValueFromPref as Boolean) {
            binding.container.bannerContainer.visibility = View.GONE
        } else {
            /*admob*/
            AdController.loadBannerAd(activity, binding.container.bannerContainer)
            AdController.loadInterAd(activity)
            binding.container.bannerContainer.visibility = View.VISIBLE
        }

        method = Method(this)
        GlobalBus.getBus().register(this)

        position = intent.getIntExtra("position", 0)
        type = intent.getStringExtra("type")
        showArray = ArrayList()
        when (type) {
            "image" -> showArray = Constants.imageArray
            "video" -> showArray = Constants.videoArray
        }
        previewAdapter = PreviewAdapter(activity, showArray, type)
        binding.viewPager.adapter = previewAdapter
        binding.viewPager.currentItem = position

        binding.viewPager.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {
                checkIfWallpaperApplicable()

                AdController.adCounter++
                AdController.showInterAd(activity, null,0)
            }

            override fun onPageScrollStateChanged(state: Int) {}
        })

        checkIfWallpaperApplicable()

        binding.backBtn.setOnClickListener {
            onBackPressed()
        }

        binding.shareBtn.setOnClickListener {
            when (type) {
                "image" -> {
                    Utils.shareFile(
                        activity!!,
                        showArray!![binding.viewPager.currentItem].toString(),
                        "image"
                    )
                }
                "video" -> {
                    Utils.shareFile(
                        activity!!,
                        showArray!![binding.viewPager.currentItem].toString(),
                        "video"
                    )
                }
            }
        }

        binding.deleteBtn.setOnClickListener {
            if (showArray!!.isNotEmpty()) {
                Utils.deleteFile(activity!!, showArray!![binding.viewPager.currentItem].toString(),
                    showArray!!, binding.viewPager, previewAdapter!!, type!!
                )
            }
        }

        binding.repostBtn.setOnClickListener {
            when (type) {
                "image" -> {
                    Utils.repostFile(activity!!, "image", showArray!![binding.viewPager.currentItem].toString())
                }
                "video" -> {
                    Utils.repostFile(activity!!, "video", showArray!![binding.viewPager.currentItem].toString())
                }
            }
        }

        binding.wallpaperBtn.setOnClickListener {
            CropImage.activity(Uri.fromFile(File(showArray!![binding.viewPager.currentItem].toString()))).start(activity!!)
        }

    }

    @Subscribe
    fun getNotify(adapterNotify: Events.AdapterNotify?) {
        if (previewAdapter != null) {
            previewAdapter!!.notifyDataSetChanged()
        }
    }

    private fun checkIfWallpaperApplicable() {
        if (type == "image") {
            binding.wallpaperBtn.visibility = View.VISIBLE
        } else {
            binding.wallpaperBtn.visibility = View.GONE
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == CropImage.CROP_IMAGE_ACTIVITY_REQUEST_CODE) {
            val result: CropImage.ActivityResult = CropImage.getActivityResult(data)
            if (resultCode == RESULT_OK) {
                val resultUri: Uri = result.uri
                try {
                    val myBitmap =
                        BitmapFactory.decodeStream(contentResolver.openInputStream(resultUri))
                    val myWallpaperManager = WallpaperManager.getInstance(
                        applicationContext
                    )
                    myWallpaperManager.setBitmap(myBitmap)
                } catch (e: IOException) {
                    Log.d("data_app", e.toString())
                }
            } else if (resultCode == CropImage.CROP_IMAGE_ACTIVITY_RESULT_ERROR_CODE) {
                val error: Exception = result.error
                Log.d("data_app", error.toString())
            }
        }
    }

    override fun onDestroy() {
        GlobalBus.getBus().unregister(this)
        super.onDestroy()
    }

    private val preferenceObject: SharedPreferences
        get() = applicationContext.getSharedPreferences(PremiumActivity.PREF_FILE, 0)

    override fun onBackPressed() {
        if (PremiumActivity.purchaseValueFromPref as Boolean) {
            super.onBackPressed()
        } else {
            AdController.adCounter++
            if (AdController.adCounter == AdController.adDisplayCounter) {
                AdController.showInterAd(activity, null, 0)
            } else {
                super.onBackPressed()
            }

        }

    }

}