package com.upscapesoft.instasaverapp.fragments

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.AsyncTask
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.ButterKnife
import com.upscapesoft.instasaverapp.activities.PreviewActivity
import com.upscapesoft.instasaverapp.adapters.FileListAdapter
import com.upscapesoft.instasaverapp.data.Constants
import com.upscapesoft.instasaverapp.databinding.FragmentVideosBinding
import com.upscapesoft.instasaverapp.helper.Events
import com.upscapesoft.instasaverapp.helper.GlobalBus
import com.upscapesoft.instasaverapp.helper.Method
import com.upscapesoft.instasaverapp.interfaces.OnClick
import org.greenrobot.eventbus.Subscribe
import java.io.File
import java.util.*

class VideosFragment : Fragment() {

    private var activity: Activity? = null
    private lateinit var binding: FragmentVideosBinding

    private var method: Method? = null
    private var onClick: OnClick? = null
    private var listAdapter: FileListAdapter? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentVideosBinding.inflate(layoutInflater)
        val root: View = binding.root
        ButterKnife.bind(this, root)
        activity = getActivity()

        contentViews()

        return root
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        activity = context as Activity
    }

    private fun contentViews() {
        GlobalBus.getBus().register(this)

        binding.progressLoader.visibility = View.GONE

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            binding.recyclerView.setPadding(0, 610, 0, 0)
            binding.noFilesLay.setPadding(0, 30, 0, 0)
        }

        binding.recyclerView.setHasFixedSize(true)
        val layoutManager: RecyclerView.LayoutManager = GridLayoutManager(getActivity(), 3)
        binding.recyclerView.layoutManager = layoutManager

        onClick = OnClick { position, type, data ->
            startActivity(
                Intent(getActivity(), PreviewActivity::class.java)
                    .putExtra("position", position)
                    .putExtra("type", type)
            )
        } as OnClick
        method = Method(getActivity(), onClick)

    }

    override fun onResume() {
        Video().execute()
        super.onResume()
    }

    @SuppressLint("StaticFieldLeak")
    private inner class Video : AsyncTask<String?, String?, String?>() {
        var files: Queue<File>? = null
        override fun onPreExecute() {
            super.onPreExecute()
            binding.progressLoader.visibility = View.VISIBLE
            if (method!!.download.listFiles() != null) {
                files = LinkedList(Arrays.asList(*method!!.download.listFiles()!!))
            }
            Constants.videoArray.clear()
        }

        override fun doInBackground(vararg strings: String?): String? {
            try {
                while (!files!!.isEmpty()) {
                    val file = files!!.remove()
                    if (file.isDirectory) {
                        files!!.addAll(Arrays.asList(*file.listFiles()!!))
                    } else if (file.name.endsWith(".mp4")) {
                        Constants.videoArray.add(file)
                    }
                }
            } catch (e: Exception) {
                Log.d("error", e.toString())
            }
            Collections.sort(Constants.videoArray)
            Collections.reverse(Constants.videoArray)
            return null
        }

        override fun onPostExecute(s: String?) {
            if (Constants.videoArray.size == 0) {
                binding.noFilesLay.visibility = View.VISIBLE
            } else {
                setVideoData()
            }
            binding.progressLoader.visibility = View.GONE
            super.onPostExecute(s)
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    @Subscribe
    fun getNotify(adapterNotify: Events.AdapterNotify?) {
        if (listAdapter != null) {
            listAdapter!!.notifyDataSetChanged()
        } else {
            setVideoData()
        }
    }

    private fun setVideoData() {
        binding.noFilesLay.visibility = View.GONE
        listAdapter = FileListAdapter(getActivity(), Constants.videoArray, "video", onClick)
        binding.recyclerView.adapter = listAdapter
    }

    override fun onDestroyView() {
        super.onDestroyView()
        GlobalBus.getBus().unregister(this)
    }

}