package com.upscapesoft.instasaverapp.services;

import android.annotation.SuppressLint;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.media.MediaScannerConnection;
import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.RemoteViews;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.NotificationCompat;

import com.upscapesoft.instasaverapp.R;
import com.upscapesoft.instasaverapp.data.Constants;
import com.upscapesoft.instasaverapp.helper.Events;
import com.upscapesoft.instasaverapp.helper.GlobalBus;
import com.upscapesoft.instasaverapp.helper.Method;

import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import io.github.lizhangqu.coreprogress.ProgressHelper;
import io.github.lizhangqu.coreprogress.ProgressUIListener;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.BufferedSink;
import okio.BufferedSource;
import okio.Okio;

public class DownloadService extends Service {

    private int position = 0;
    private RemoteViews rv;
    private OkHttpClient client;
    private final int CHANEL_ID = 124;
    private NotificationCompat.Builder builder;
    private NotificationManager notificationManager;
    private static final String CANCEL_TAG = "c_tag";
    private final String NOTIFICATION_CHANNEL_ID = "download_service";
    public static final String ACTION_START = "com.download.action.START";
    public static final String ACTION_STOP = "com.download.action.STOP";

    private final Handler mHandler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(@NotNull Message message) {
            int progress = Integer.parseInt(message.obj.toString());
            switch (message.what) {
                case 1:
                    rv.setTextViewText(R.id.nf_title, getString(R.string.app_name));
                    rv.setProgressBar(R.id.progress, 100, progress, false);
                    rv.setTextViewText(R.id.nf_percentage, "Downloading" + " " + "(" + progress + " %)");
                    notificationManager.notify(CHANEL_ID, builder.build());
                    break;
                case 2:
                    if (Constants.downloadArray.size() - 1 != position) {
                        position++;
                        init(Constants.downloadArray.get(position));
                    } else {
                        position = 0;
                        stopForeground(false);
                        stopSelf();
                        Events.AdapterNotify adapterNotify = new Events.AdapterNotify("");
                        GlobalBus.getBus().post(adapterNotify);
                        Method.isDownload = true;
                    }
                    break;
            }
            return false;
        }
    });

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        builder = new NotificationCompat.Builder(this, NOTIFICATION_CHANNEL_ID);
        builder.setChannelId(NOTIFICATION_CHANNEL_ID);
        builder.setSmallIcon(R.drawable.app_icon);
        builder.setTicker("Downloading...");
        builder.setWhen(System.currentTimeMillis());
        builder.setOnlyAlertOnce(true);

        rv = new RemoteViews(getPackageName(), R.layout.file_download_notification_lay);
        rv.setTextViewText(R.id.nf_title, getString(R.string.app_name));
        rv.setProgressBar(R.id.progress, 100, 0, false);
        rv.setTextViewText(R.id.nf_percentage, "Downloading" + " " + "(0%)");

        Intent intentClose = new Intent(this, DownloadService.class);
        intentClose.setAction(ACTION_STOP);
        PendingIntent closeIntent = PendingIntent.getService(this, 0, intentClose, PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
        rv.setOnClickPendingIntent(R.id.nf_close, closeIntent);

        builder.setCustomContentView(rv);
        NotificationChannel mChannel;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            CharSequence name = getResources().getString(R.string.app_name);
            mChannel = new NotificationChannel(NOTIFICATION_CHANNEL_ID, name, NotificationManager.IMPORTANCE_HIGH);
            notificationManager.createNotificationChannel(mChannel);
        }
        startForeground(CHANEL_ID, builder.build());
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        stopForeground(false);
        stopSelf();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId);
        try {
            if (intent.getAction() != null && intent.getAction().equals(ACTION_START)) {
                Method.isDownload = false;
                init(Constants.downloadArray.get(position));
            }
            if (intent.getAction() != null && intent.getAction().equals(ACTION_STOP)) {
                if (client != null) {
                    for (Call call : client.dispatcher().runningCalls()) {
                        if (call.request().tag().equals(CANCEL_TAG))
                            call.cancel();
                    }
                }
                Method.isDownload = true;
                stopForeground(false);
                stopSelf();
            }
        } catch (Exception e) {
            Log.d("error", e.toString());
            stopForeground(false);
            stopSelf();
        }
        return START_STICKY;
    }

    public void init(final String downloadUrl) {
        final String storagePath = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)
                .toString() + File.separator + getResources().getString(R.string.app_name) + File.separator;

        Date date = new Date();
        @SuppressLint("SimpleDateFormat")
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
        String stringDate = sdf.format(date);

        Calendar cal = Calendar.getInstance();
        String s = sdf.format(cal.getTime());

        final String string;

        if (downloadUrl.contains(".jpg") || downloadUrl.contains(".webp")) {
            string = "Image_" + s.replace(":", "") + ".jpg";
        } else {
            string = "Video_" + s.replace(":", "") + ".mp4";
        }
        Log.d("file_name", string);

        new Thread(new Runnable() {
            @Override
            public void run() {
                client = new OkHttpClient();
                Request.Builder builder = new Request.Builder()
                        .url(downloadUrl)
                        .addHeader("Accept-Encoding", "identity")
                        .get()
                        .tag(CANCEL_TAG);

                Call call = client.newCall(builder.build());

                call.enqueue(new Callback() {
                    @Override
                    public void onFailure(@NonNull Call call, @NonNull IOException e) {
                        Log.e("TAG", "=============onFailure===============");
                        e.printStackTrace();
                        Log.d("error_downloading", e.toString());
                    }

                    @Override
                    public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                        Log.e("TAG", "=============onResponse===============");
                        Log.e("TAG", "request headers:" + response.request().headers());
                        Log.e("TAG", "response headers:" + response.headers());
                        assert response.body() != null;
                        ResponseBody responseBody = ProgressHelper.withProgress(response.body(), new ProgressUIListener() {

                            @Override
                            public void onUIProgressStart(long totalBytes) {
                                super.onUIProgressStart(totalBytes);
                                Log.e("TAG", "onUIProgressStart:" + totalBytes);
                                Toast.makeText(getApplicationContext(), "Downloading file...", Toast.LENGTH_SHORT).show();
                            }

                            @Override
                            public void onUIProgressChanged(long numBytes, long totalBytes, float percent, float speed) {
                                Log.e("TAG", "=============start===============");
                                Log.e("TAG", "numBytes:" + numBytes);
                                Log.e("TAG", "totalBytes:" + totalBytes);
                                Log.e("TAG", "percent:" + percent);
                                Log.e("TAG", "speed:" + speed);
                                Log.e("TAG", "============= end ===============");
                                Message msg = mHandler.obtainMessage();
                                msg.what = 1;
                                msg.obj = (int) (100 * percent) + "";
                                mHandler.sendMessage(msg);
                            }

                            @Override
                            public void onUIProgressFinish() {
                                super.onUIProgressFinish();
                                Log.e("TAG", "onUIProgressFinish:");
                                Message msg = mHandler.obtainMessage();
                                msg.what = 2;
                                msg.obj = 0 + "";
                                mHandler.sendMessage(msg);
                                Toast.makeText(getApplicationContext(), "File downloaded", Toast.LENGTH_SHORT).show();
                                if (string.contains(".jpg")) {
                                    if (Constants.imageArray != null) {
                                        Constants.imageArray.add(0, new File(storagePath + "/" + string));
                                    }
                                } else {
                                    if (Constants.videoArray != null) {
                                        Constants.videoArray.add(0, new File(storagePath + "/" + string));
                                    }
                                }
                                try {
                                    MediaScannerConnection.scanFile(getApplicationContext(), new String[]{storagePath + "/" + string},
                                            null,
                                            (path, uri) -> {

                                            });
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }
                        });

                        try {

                            BufferedSource source = responseBody.source();
                            File outFile = new File(storagePath + "/" + string);
                            BufferedSink sink = Okio.buffer(Okio.sink(outFile));
                            source.readAll(sink);
                            sink.flush();
                            source.close();

                        } catch (Exception e) {
                            Log.d("show_data", e.toString());
                        }

                    }
                });
            }
        }).start();
    }

}
