package com.upscapesoft.instasaverapp.utils

import android.Manifest
import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.ClipData
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.util.TypedValue
import android.widget.Toast
import androidx.annotation.AttrRes
import androidx.annotation.ColorInt
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import androidx.viewpager.widget.ViewPager
import com.upscapesoft.instasaverapp.BuildConfig
import com.upscapesoft.instasaverapp.R
import com.upscapesoft.instasaverapp.adapters.PreviewAdapter
import com.upscapesoft.instasaverapp.helper.Events
import com.upscapesoft.instasaverapp.helper.GlobalBus
import java.io.File

class Utils(private var context: Context) {

    companion object {
        var permissions = arrayOf(
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE
        )
        var permissionRequest = 21

        fun hasPermissions(context: Context?, vararg permissions: String?): Boolean {
            if (permissions != null) {
                for (permission in permissions) {
                    if (ContextCompat.checkSelfPermission(
                            context!!,
                            permission!!
                        ) != PackageManager.PERMISSION_GRANTED
                    ) {
                        return true
                    }
                }
            }
            return false
        }

        @ColorInt
        fun Context.getColorFromAttr(
            @AttrRes attrColor: Int,
            typedValue: TypedValue = TypedValue(),
            resolveRefs: Boolean = true
        ): Int {
            theme.resolveAttribute(attrColor, typedValue, resolveRefs)
            return typedValue.data
        }

        fun isPackageInstalled(packageName: String, packageManager: PackageManager): Boolean {
            return try {
                packageManager.getPackageInfo(packageName, 0)
                true
            } catch (e: PackageManager.NameNotFoundException) {
                false
            }
        }

        fun shareFile(activity: Activity, link: String?, type: String) {
            val contentUri = FileProvider.getUriForFile(
                activity,
                BuildConfig.APPLICATION_ID + ".fileprovider",
                File(link)
            )
            val shareIntent = Intent()
            shareIntent.action = Intent.ACTION_SEND
            if (type == "image") {
                shareIntent.type = "image/*"
            } else {
                shareIntent.type = "video/*"
            }
            shareIntent.putExtra(Intent.EXTRA_STREAM, contentUri)
            shareIntent.clipData = ClipData.newRawUri("", contentUri)
            shareIntent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION or Intent.FLAG_GRANT_READ_URI_PERMISSION)
            activity.startActivity(
                Intent.createChooser(
                    shareIntent,
                    activity.resources.getString(R.string.share_via)
                )
            )
        }

        fun repostFile(activity: Activity, type: String, link: String) {
            try {
                val contentUri = FileProvider.getUriForFile(
                    activity,
                    BuildConfig.APPLICATION_ID + ".fileprovider",
                    File(link)
                )
                val mIntent = Intent("android.intent.action.SEND")
                if (type == "image") {
                    mIntent.type = "image/*"
                } else {
                    mIntent.type = "video/*"
                }
                mIntent.setPackage("com.instagram.android")
                mIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                mIntent.clipData = ClipData.newRawUri("", contentUri)
                mIntent.putExtra("android.intent.extra.STREAM", contentUri)
                activity.startActivity(mIntent)
            } catch (e: ActivityNotFoundException) {
                Toast.makeText(activity, "Instagram app is not installed. Please install!", Toast.LENGTH_LONG).show()
            }
        }

        fun deleteFile(activity: Activity, link: String, showArray: MutableList<File>, viewPager: ViewPager, previewAdapter: PreviewAdapter, type: String) {
            try {
                val files = File(link)
                files.delete()
                showArray.removeAt(viewPager.currentItem)
                previewAdapter.notifyDataSetChanged()
                if (showArray.size == 0) {
                    activity.onBackPressed()
                }
                val adapterNotify = Events.AdapterNotify(type)
                GlobalBus.getBus().post(adapterNotify)
                Toast.makeText(
                    activity,
                    "File deleted successfully",
                    Toast.LENGTH_SHORT
                ).show()
            } catch (e: java.lang.Exception) {
                Toast.makeText(activity, "Something went wrong!", Toast.LENGTH_LONG).show()
            }
        }

        fun shareApp(activity: Activity, appName: String) {
            val sharingIntent = Intent(Intent.ACTION_SEND)
            sharingIntent.type = "text/plain"
            val shareBodyText =
                "https://play.google.com/store/apps/details?id=" + activity.packageName
            sharingIntent.putExtra(
                Intent.EXTRA_SUBJECT,
                "Want to extract text from images? Try $appName. DOWNLOAD NOW!"
            )
            sharingIntent.putExtra(Intent.EXTRA_TEXT, shareBodyText)
            activity.startActivity(
                Intent.createChooser(
                    sharingIntent,
                    activity.resources.getString(R.string.share_via)
                )
            )
        }

        fun rateApp(activity: Activity) {
            activity.startActivity(
                Intent(
                    Intent.ACTION_VIEW,
                    Uri.parse("market://details?id=" + activity.packageName)
                )
            )
        }

        fun moreApps(activity: Activity) {
            activity.startActivity(
                Intent(
                    Intent.ACTION_VIEW,
                    Uri.parse("market://details?id=" + activity.packageName)
                )
            )
        }

        fun openUrl(activity: Activity, url: String) {
            if (url.isEmpty()) return
            val intent = Intent(Intent.ACTION_VIEW)
            intent.data = Uri.parse(url)
            activity.startActivity(intent)
        }

    }

}